<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

$plugin_info = array(
						'pi_name'			=> 'MaxMind Geoloc',
						'pi_version'		=> '1.0.1',
						'pi_author'			=> 'Matthieu Fauveau',
						'pi_author_url'		=> 'http://www.matthieufauveau.com/',
						'pi_description'	=> 'Allows you to display data from MaxMind City/ISP/Org web service API',
						'pi_usage'			=> Maxmind_geoloc::usage()
					);
					
/**
 * Maxmind_geoloc Class
 *
 * @package			ExpressionEngine
 * @category		Plugin
 * @author			Matthieu Fauveau
 * @copyright		Copyright (c) 2010, Matthieu Fauveau.
 * @link			http://expressionengine.com/forums/viewthread/156992/
 */
class Maxmind_geoloc {
	
	var $return_data;
	
	var $data			= array();
	var $data_keys		= array('country_code', 'region', 'city', 'postal_code', 'latitude', 'longitude', 'metro_code', 'area_code', 'isp', 'organization');
	
	var $base_url		= 'http://geoip3.maxmind.com/';
	
	var $cache_name		= 'maxmind_geoloc_cache';
	var $refresh		= 15; // period between cache refreshes, in minutes
	
	var $license_key	= '';
	var $ip				= '';
		
	/**
	 * Constructor
	 *
	 */
	function Maxmind_geoloc()
	{
		$this->EE =& get_instance();
		
		$this->return_data = $this->EE->TMPL->tagdata;
		
		/** ---------------------------------------
		/**  Fetch parameters
		/** ---------------------------------------*/
		
		$this->license_key = (($license_key = $this->EE->TMPL->fetch_param('license_key')) === FALSE) ? $this->license_key : $license_key;
		$this->ip = (($ip = $this->EE->TMPL->fetch_param('ip')) === FALSE) ? $this->EE->input->ip_address() : $ip;
		
		if ($this->license_key == '') {
			$this->EE->TMPL->log_item("MaxMind Geoloc error: no license specified.");
			$this->return_data = '';
			return;
		}
		
		/** ---------------------------------------
		/**  Fetch data
		/** ---------------------------------------*/
		
		require("php_api/GeoCityLocateIspOrg.class.php");
		
		if (($this->data = unserialize($this->_check_cache($this->ip))) === FALSE) {
			$this->EE->TMPL->log_item("Fetching MaxMind Geoloc data remotely");
			
			$geoloc = GeoCityLocateIspOrg::getInstance();
			$geoloc->setLicenceKey($this->license_key);
			$geoloc->setIP($this->ip);
			
			if ($geoloc->isError()) {
				$this->EE->TMPL->log_item("MaxMind Geoloc error: ".$geoloc->getError());
				$this->return_data = '';
				return;
			}
			else {
				$this->data = $geoloc->getResultArray();
				
				$this->_write_cache(serialize($this->data), $this->ip);	
			}
		}
		
		/** ---------------------------------------
		/**  Go go go!
		/** ---------------------------------------*/

		foreach ($this->data_keys as $key => $val) {
			if (isset($this->data[$key])) {
				$this->return_data = $this->EE->TMPL->swap_var_single($val, $this->data[$key], $this->return_data);
			}
			else {
				$this->return_data = $this->EE->TMPL->swap_var_single($val, '', $this->return_data);
			}
		} 
		
		return $this->return_data;
	}

	// --------------------------------------------------------------------
	
	/**
	 * Check Cache
	 *
	 * Check for cached data
	 *
	 * @param string	$url
	 *
	 * @access public
	 * @return mixed	string if pulling from cache, FALSE if not
	 */
	function _check_cache($url)
	{	
		/** ---------------------------------------
		/**  Check for cache directory
		/** ---------------------------------------*/
	
		$dir = APPPATH.'cache/'.$this->cache_name.'/';
		
		if ( ! @is_dir($dir))
		{
			return FALSE;
		}

        $file = $dir.md5($url);
		
		if ( ! file_exists($file) OR ! ($fp = @fopen($file, 'rb')))
		{
			return FALSE;
		}
		       
		flock($fp, LOCK_SH);
                    
		$cache = @fread($fp, filesize($file));
                    
		flock($fp, LOCK_UN);
        
		fclose($fp);
        
		$eol = strpos($cache, "\n");
		
		$timestamp = substr($cache, 0, $eol);
		$cache = trim((substr($cache, $eol)));
		
		if (time() > ($timestamp + ($this->refresh * 60)))
		{
			return FALSE;
		}
		
		$this->EE->TMPL->log_item("MaxMind data retrieved from cache");
		
        return $cache;
	}

	// --------------------------------------------------------------------
	
	/**
	 * Write Cache
	 *
	 * Write the cached data
	 *
	 * @param string	$data
	 * @param string	$url
	 *
	 * @access public
	 * @return	void
	 */
	function _write_cache($data, $url)
	{
		/** ---------------------------------------
		/**  Check for cache directory
		/** ---------------------------------------*/
		
		$dir = APPPATH.'cache/'.$this->cache_name.'/';

		if ( ! @is_dir($dir))
		{
			if ( ! @mkdir($dir, 0777))
			{
				return FALSE;
			}
			
			@chmod($dir, 0777);            
		}
		
		// add a timestamp to the top of the file
		$data = time()."\n".$data;
		
		/** ---------------------------------------
		/**  Write the cached data
		/** ---------------------------------------*/
		
		$file = $dir.md5($url);
	
		if ( ! $fp = @fopen($file, 'wb'))
		{
			return FALSE;
		}

		flock($fp, LOCK_EX);
		fwrite($fp, $data);
		flock($fp, LOCK_UN);
		fclose($fp);
        
		@chmod($file, 0777);	
	}

	// --------------------------------------------------------------------
	
	/**
	 * Usage
	 *
	 * Plugin Usage
	 *
	 * @access public
	 * @return string
	 */
	function usage()
	{
		ob_start(); 
		?>
		------------------
		EXAMPLE USAGE:
		------------------
		{exp:maxmind_geoloc license_key='XXXXXXXXXXXX'}
		Country code: {country_code}<br />
		Region: {region}<br />
		City: {city}<br />
		Postal Code: {postal_code}<br />
		Latitude: {latitude}<br />
		Longitude: {longitude}<br />
		Metro Code: {metro_code}<br />
		Area Code: {area_code}<br />
		ISP: {isp}<br />
		Organization: {organization}
		{/exp:maxmind_geoloc}

		------------------
		PARAMETERS:
		------------------

		license_key=""
		- Unfortunately a license key is required to use MaxMind Web Service. If one is not provided the plugin return empty data.

		ip=""
		- Allows you to set the IP manually, otherwise it's "discovered" by ExpressionEngine.
		
		------------------
		VARIABLES:
		------------------
		
		See example usage.

		------------------
		CHANGELOG:
		------------------

		Version 1.0.1	- Tags were not replaced when the data were empty
		Version 1.0 	- Initial release
				
		------------------
		DEVELOPERS:
		------------------
		
		Note that this plugin use a slightly modified version of the MaxMind's PHP API by Nathan White. That said, it's really only 
		to avoid PHP warnings.
		
		All in all, this could be easily improved but I just needed it to grab the city. Feel free to carry on.
		
		<?php
		$buffer = ob_get_contents();

		ob_end_clean(); 

		return $buffer;
	}

	// --------------------------------------------------------------------
	
}
// END Maxmind_geoloc Class

/* End of file  pi.maxmind_geoloc.php */
/* Location: ./system/expressionengine/third_party/maxmind_geoloc/pi.maxmind_geoloc.php */